@@ -6,6 +6,7 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <stdlib.h>
 
 #ifndef _WIN32
 #include <sys/uio.h>
@@ -726,13 +727,18 @@ SEXP processx_connection_disable_inheritance() {
 
 #else
 
-SEXP processx_connection_disable_inheritance() {
+SEXP processx_connection_disable_inheritance(void) {
   int fd;
 
   /* Set the CLOEXEC flag on all open descriptors. Unconditionally try the
    * first 16 file descriptors. After that, bail out after the first error.
-   */
-  for (fd = 0; ; fd++) {
+   * We skip the standard streams, because R and `system()` is not prepared
+   * to not inheriting stdin and eg. an R subprocess does not even start in
+   * system(). See https://github.com/r-lib/callr/issues/236. */
+
+  int firstfd = 3;
+  if (getenv("PROCESSX_CLOEXEC_STDIO")) firstfd = 0;
+  for (fd = firstfd; ; fd++) {
     if (processx__cloexec_fcntl(fd, 1) && fd > 15) break;
   }
 
